package hdm.mi.sthbackend;

import com.fasterxml.jackson.databind.ObjectMapper;
import hdm.mi.sthbackend.controller.TournamentController;
import hdm.mi.sthbackend.model.BracketRound;
import hdm.mi.sthbackend.model.Match;
import hdm.mi.sthbackend.model.Tournament;
import hdm.mi.sthbackend.repository.ITournamentRepository;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;

import java.util.ArrayList;
import java.util.Optional;
import java.util.UUID;

import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.patch;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.post;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest
@AutoConfigureMockMvc
@ExtendWith(MockitoExtension.class)
public class TournamentControllerTest {
    private final Logger log = LogManager.getLogger("TournamentControllerTest");

    @Mock
    ITournamentRepository tournamentRepository;
    @InjectMocks
    TournamentController tournamentController;
    @Autowired
    public MockMvc mockMvc;
    @Test
    public void assignTeamToMatch() throws Exception {

        UUID tournamentId = UUID.randomUUID();
        UUID matchId = UUID.randomUUID();
        UUID teamId = UUID.randomUUID();
        Match match = new Match(matchId);
        Tournament tournament = new Tournament("TestTournament", new ArrayList<>());
        tournament.getBracket().add(new BracketRound(0));
        tournament.getBracket().get(0).getMatches().put(matchId, match);

        when(tournamentRepository.findById(tournamentId)).thenReturn(Optional.of(tournament));

        mockMvc.perform(patch("/tournaments/{tournamentId}/matches/{matchId}/teams/{teamId}/assignTeamToMatch", tournamentId, matchId, teamId)
                        .contentType(MediaType.APPLICATION_JSON)
                        .content("{\"bracketRound\": 0, \"teamMatchScore\": 0}"))
                .andExpect(status().isOk());


        verify(tournamentRepository, times(1)).findById(tournamentId);
        verify(tournamentRepository, times(1)).save(tournament);
    }

    @Test
    public void createTournamentTest() throws Exception {

        UUID tournamentId = UUID.randomUUID();
        Tournament tournament = new Tournament("TestTournament", new ArrayList<>());
        when(tournamentRepository.findById(tournamentId)).thenReturn(Optional.of(tournament));

        mockMvc.perform(post("/tournaments")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(new ObjectMapper().writeValueAsString(tournament)))
                .andExpect(status().isOk());
    }
}
